/*
 * Copyright (C) 2002-2010 Edscott Wilson Garcia
 * EMail: edscott@xfce.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <dirent.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "rfm_libs.h"
#include "rfm_modules.h"


/**
 * rfm_copy_entry:
 * @src_en: a valid record_entry_t pointer
 * Returns: allocated record_entry_t pointer with same information as src_en
 *
 * Allocates memory and copies the input record entry. 
 *
 **/
record_entry_t *
rfm_copy_entry (record_entry_t * src_en) {
    record_entry_t *en;
    if(!src_en)
        return NULL;
    en = (record_entry_t *) malloc (sizeof (record_entry_t));
    memset (en, 0, sizeof (record_entry_t));
    if(!en)
        g_assert_not_reached ();
    memcpy (en, src_en, sizeof (record_entry_t));

    if(src_en->st) {
        en->st = (struct stat *)malloc (sizeof (struct stat));
        memcpy (en->st, src_en->st, sizeof (struct stat));
    }

    en->module = (src_en->module) ? src_en->module : NULL;
    en->parent_module = (src_en->parent_module) ? src_en->parent_module : NULL;
    en->tag = (src_en->tag) ? g_strdup (src_en->tag) : NULL;
    en->path = (src_en->path) ? g_strdup (src_en->path) : NULL;

    en->filetype = (src_en->filetype) ? g_strdup (src_en->filetype) : NULL;
    en->mimetype = (src_en->mimetype) ? g_strdup (src_en->mimetype) : NULL;
    en->mimemagic = (src_en->mimemagic) ? g_strdup (src_en->mimemagic) : NULL;
    en->encoding = (src_en->encoding) ? g_strdup (src_en->encoding) : NULL;


    return en;
}

/**
 * rfm_destroy_entry:
 * @en: a valid record_entry_t pointer
 *
 * Frees allocated memory for the record entry.
 *
 **/
void
rfm_destroy_entry (record_entry_t * en) {
    if(!en)
        return;
    TRACE ("thread monitor: will free en=0x%lx\n", (long unsigned)en);
// this is const:  g_free (en->module);
    g_free (en->tag);
    g_free (en->path);

    g_free (en->filetype);
    g_free (en->mimetype);
    g_free (en->mimemagic);
    g_free (en->encoding);


    g_free (en->st);
    g_free (en);
    en = NULL;
}

/**
 * rfm_mk_entry:
 * @type: Type of record entry to inherit. May be 0.
 * Returns: allocated record_entry_t pointer
 *
 * Allocates memory for an empty record entry inheriting @type characteristics.
 **/
record_entry_t *
rfm_mk_entry (int type) {
    record_entry_t *en;
    en = (record_entry_t *) malloc (sizeof (record_entry_t));
    if(!en)
        g_assert_not_reached ();
    memset (en, 0, sizeof (record_entry_t));
    INHERIT_TYPE (en->type, type);
    return en;
}

/**
 * mk_entry_path:
 * @path: path to local file to obtain stat information.
 * @type: Type of record entry to inherit. May be 0.
 * Returns: allocated record_entry_t pointer
 *
 * Allocates memory without filling in stat information for a record entry 
 * pointing to @path. Type may be inherited from @type if different
 * from 0.
 **/
static
record_entry_t *
mk_entry_path (const gchar * path, int type) {
    record_entry_t *en;
    en = rfm_mk_entry (type);
    en->path = g_strdup (path);
    if(access (path, W_OK) < 0) {
        SET_NOWRITE_TYPE (en->type);
    }
    if(access (path, R_OK) < 0) {
        SET_NOACCESS_TYPE (en->type);
    }
    if(access (path, X_OK) >= 0) {
        /* this fails for priviledged users so retest */
        struct stat s;
        if(stat (path, &s) != -1 && ((s.st_mode & S_IXUSR) || (s.st_mode & S_IXGRP) || (s.st_mode & S_IXOTH))) {
            SET_EXE_TYPE (en->type);
        }
    }
    return en;
}

/**
 * rfm_stat_entry:
 * @path: path to local file to obtain stat information.
 * @type: Type of record entry to inherit. May be 0.
 * Returns: allocated record_entry_t pointer
 *
 * Allocates memory and fills in stat information for a record entry 
 * pointing to @path. Type may be inherited from @type if different
 * from 0.
 *
 **/
static
void
entry_set_type(record_entry_t *en){
    struct stat *st=en->st;
    if(S_ISDIR (st->st_mode)) {
	SET_SDIR (en->type);
    } else if(S_ISSOCK (st->st_mode)) {
        SET_SSOCK (en->type);
    } else if(S_ISBLK (st->st_mode)) {
        SET_SBLK (en->type);
    } else if(S_ISCHR (st->st_mode)) {
        SET_SCHR (en->type);
    } else if(S_ISFIFO (st->st_mode)) {
        SET_SFIFO (en->type);
    } else {
        SET_SREG (en->type);
    }

    //if(g_file_test (en->path, G_FILE_TEST_IS_EXECUTABLE))
    if(st->st_mode & (S_IXUSR|S_IXGRP|S_IXOTH) && S_ISREG(st->st_mode)) {
        SET_EXE_TYPE (en->type);
    }
    if(access (en->path, R_OK) < 0) {
        SET_NOACCESS_TYPE (en->type);
    }
    if(access (en->path, W_OK) != 0) {
        //if(g_file_test (en->path, G_FILE_TEST_IS_DIR)) {
        if(S_ISDIR (st->st_mode)) {
            SET_NOWRITE_TYPE (en->type);
        } else {
            /* if you can write to the directory you can
             * write to the file. Let us check... */
            gchar *g = g_path_get_dirname (en->path);
            if(access (g, W_OK) != 0) {
                SET_NOWRITE_TYPE (en->type);
            }
            g_free (g);
        }
    }

}

record_entry_t *
rfm_stat_entry (const gchar * path, int type) {
    record_entry_t *en = mk_entry_path (path, type);
    struct stat *st;
    st = (struct stat *)malloc (sizeof (struct stat));
    st = memset(st, 0,sizeof(struct stat));
    en->st = st;

    /*SET_LOCAL_TYPE(en->type);  default */

    if(!st){
        g_assert_not_reached ();
    } else if(lstat (path, st) < 0) {
            g_warning ("primary-entry.c: 198, cannot lstat %s", en->path);
            memset (st, 0, sizeof (struct stat));
    } 
    if(lstat (path, st) < 0) {
            g_warning ("primary-entry.c: 198, cannot lstat %s", en->path);
            memset (st, 0, sizeof (struct stat));
    } else {
	if (S_ISLNK(st->st_mode)) {
	    if(stat (path, st) < 0) {
		SET_SLNK(en->type);
		SET_BROKEN_LNK (en->type);
		return en;
	    } 
	    SET_SLNK(en->type);
	}
    }
    en->st = st;
    entry_set_type(en);
    return en;
}

/**
 * rfm_set_entry_tag:
 * @src_en: a valid record_entry_t pointer
 * @tama: Size in bytes.
 * 
 * Sets the entry's tag (which will show up on the status line).
 *
 **/
void
rfm_set_entry_tag (record_entry_t * en, off_t tama) {
    int hcount = rfm_xfdir_count_hidden_files (en->path);
    int count = rfm_xfdir_count_files (en->path);
    gchar *b = g_path_get_basename (en->path);
    gchar *q = rfm_utf_string (b);
    g_free (b);
    gchar *s = rfm_sizetag (tama, count);
    g_free (en->tag);

    if (hcount) {
        gchar *plural_string = g_strdup_printf(ngettext ("%'u item","%'u items",hcount), hcount);
        en->tag = g_strdup_printf ("%s (%s %s: %s)", q, s, _("Hidden"), plural_string);
        g_free(plural_string);
    } else {
        en->tag = g_strdup_printf ("%s (%s)", q, s);
    }
    
    g_free (s);
}
