/*
 * Copyright (C) 2002-2010 Edscott Wilson Garcia
 * EMail: edscott@xfce.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <dirent.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include "rfm_libs.h"
#include "rfm_modules.h"

#include "primary-xfdir.i"

/**
 * SECTION:xfdir
 * @short_description: xfdir structure is a location to place the contents of 
 * the rodent view. The primary xfdir functions allow for the loading and
 * monitoring of local files on a POSIX compliant system.
 * 
 * @section_id:	xfdir
 * @stability: Current
 * @include: primary.h
 *
 **/

/**
 * xfdir_get_local:
 * @view_p: A Rodent view_t pointer
 * @en: An icon entry structure.
 *
 * Generalized xfdir load routine. en->path  should be a valid
 * path or else en->module defined so that module  particulars 
 * are used.
 *
 * This is main thread executed.
 **/
xfdir_t *
rfm_xfdir_get (view_t * view_p, record_entry_t * en) {
    
    // set the dtfile quick dir buttons (or menu items in our case)
    if(!view_p->module) {
        if(!en || !en->path || !g_file_test (en->path, G_FILE_TEST_IS_DIR)) {
            g_warning ("Invalid entry sent to xfdir_get_local");
            return NULL;
        }
        set_deepest_dir (view_p);
    }

    // allocate memory for new xfdir structure and initialize values
    xfdir_t *xfdir_p = (xfdir_t *) malloc (sizeof (xfdir_t));
    memset ((void *)xfdir_p, 0, sizeof (xfdir_t));
    strncpy (xfdir_p->path, en->path, 1023);
    xfdir_p->path[1023] = 0;
    xfdir_p->type = en->type;
    xfdir_p->view_p = view_p;

    // read local directory information and update xfdir structure
    DBG ("starting time on update_xfdir for %s...\n", xfdir_p->path);
    update_xfdir (xfdir_p);

    // set user defined sort preferences and do the sort
    xfdir_p->ascending = view_p->preferences & SORT_ASCENDING;
    xfdir_p->sort_column = view_p->sortcolumn;
    xfdir_sort (xfdir_p);

    // set default status tag for directory
    rfm_set_entry_tag (en, xfdir_p->tama);

    // create monitor variables mutex if not created already
    if(view_p->monitor_id_mutex == NULL) {
        view_p->monitor_id_mutex = g_mutex_new ();
    }
    // start background directory monitor thread
    xfdir_start_monitor (view_p, xfdir_p);

    return xfdir_p;
}

/**
 * rfm_xfdir_exit_monitor:
 * @xfdir_p:  pointer to an xfdir_t structure
 *
 * Schedule monitor to exit as soon as possible. This function should 
 * be accessed from threads other than the monitor thread. For the monitor thread,
 * use the static exit_monitor() function instead which does not have a
 * wait condition.
 **/
void
rfm_xfdir_exit_monitor (view_t * view_p) {
    if(view_p->monitor_id_mutex == NULL) {
        view_p->monitor_id_mutex = g_mutex_new ();
    }
    g_mutex_lock (view_p->monitor_id_mutex);
    view_p->monitor_id++;
    g_mutex_unlock (view_p->monitor_id_mutex);
}

/**
 * rfm_xfdir_get_local_cache_path:
 * @directory: string with the path for which the cache path name should be retrieved
 *
 * This function returns the hash key under which the cache for the 
 * local path @directory under which the cache information is saved.
 **/
gchar *
rfm_xfdir_get_local_cache_path (gchar * directory) {
    char *cache_dir;
    gchar *cache_path = NULL;

    cache_dir = g_build_filename (g_get_user_cache_dir (), "rfm", "cache", NULL);
    if(g_mkdir_with_parents (cache_dir, 0700) < 0) {
        g_free (cache_dir);
        return NULL;
    }

    cache_path = g_build_filename (cache_dir, get_local_shm_name (directory), NULL);
    g_free (cache_dir);
    return cache_path;
}

/**
 * rfm_xfdir_count_files:
 * @path: directory path
 * Returns: file count within directory, including hidden files
 *
 * This function is non-recursive.
 **/
gint
rfm_xfdir_count_files (const gchar * path) {
    if(!g_file_test (path, G_FILE_TEST_EXISTS))
        return 0;
    if(!g_file_test (path, G_FILE_TEST_IS_DIR))
        return 0;
    DIR *directory;
    directory = opendir (path);
    if(!directory)
        return 0;
    gint count = 0;
    struct dirent *d;
    while((d = readdir (directory)) != NULL) {
        if(strcmp (d->d_name, ".") == 0)
            continue;
        if(strcmp (d->d_name, "..") == 0)
            continue;
        count++;
    }
    closedir (directory);
    return count;
}

/**
 * rfm_xfdir_count_hidden_files:
 * @path: directory path
 * Returns: hidden file count within directory
 *
 * This function is non-recursive.
 **/
gint
rfm_xfdir_count_hidden_files (const gchar * path) {
    if(!g_file_test (path, G_FILE_TEST_EXISTS))
        return 0;
    if(!g_file_test (path, G_FILE_TEST_IS_DIR))
        return 0;
    DIR *directory;
    directory = opendir (path);
    if(!directory)
        return 0;
    gint count = 0;
    struct dirent *d;
    while((d = readdir (directory)) != NULL) {
        if(strcmp (d->d_name, ".") == 0)
            continue;
        if(strcmp (d->d_name, "..") == 0)
            continue;
        if(d->d_name[0] != '.')
            continue;
        count++;
    }
    closedir (directory);
    return count;
}

// this is done threaded by the main thread or directly from
// the monitor thread if view_p->monitor_enabled is TRUE
//
// if exit_status is not set, that means view_p is still active
// this function may be invoked by active monitor threads
// as well as destroy callbacks
gpointer
rfm_xfdir_view_cleanup (gpointer data) {
  void rodent_destroy_population (view_t * view_p);
   view_t *view_p = data;
    if(!view_p->exit_status) {
	DBG("No view_p cleanup: exit_status == 0\n");
    	return NULL;
    }  
    // Cleanup go history.
    GList *tmp;
    for(tmp = view_p->go_list; tmp; tmp = tmp->next) {
	rfm_destroy_entry ((record_entry_t *) (tmp->data));
    }
    g_list_free (view_p->go_list);

    // Cleanup view population (this will block until write lock obtained)
    rfm_population_write_lock (view_p);
    rodent_destroy_population (view_p);
    view_p->population_pp = NULL;
    rfm_population_write_unlock (view_p);

    // Cleanup view
    rfm_destroy_entry (view_p->en);
    g_free (view_p->deepest_dir);

    // we keep the view pointer so that the stderr/stdout threads
    // of running programs know what to do with further io
    // XXX (this is a very small memory leak for which a have not 
    // 	    worked out an acceptable solution).
    // g_free (view_p);
    DBG ("view_p thread cleanup OK!\n");
    return NULL;
}


