/*
 *
 * (C) Edscott Wilson Garcia 2001-2010 edscott@xfce.org.
 *
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/stat.h>

#include <errno.h>
#include <limits.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <regex.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <gmodule.h>

#include "rodent_libs.h"

/* this should be first 2 lines after headers: */
G_MODULE_EXPORT LIBRFM_MODULE
#define CACHE_DIR "rfm","modules"
#define DEFAULT_THEME "Rodent"
#define FALLBACK "hicolor"
#define CACHE_CHECK_LOCK g_get_tmp_dir(),"rfm_cache_lock"
#define VALID_KDEDIR g_getenv("KDEDIR") && strlen(g_getenv("KDEDIR"))
#define KDEDIR g_getenv("KDEDIR"),"share","icons"
#define HOMEDIR  g_get_home_dir(),".icons"
#define SERIAL 41912
    
typedef struct theme_info_t {
    GHashTable *icon_hash;
} theme_info_t;

typedef struct icon_info_t {
    gchar *path;
    int size;
    gchar *type;
    struct icon_info_t *next;
} icon_info_t;

static gchar *theme_name = NULL;
static theme_info_t *theme_info = NULL;

typedef struct cache_info_t {
    gchar version[64];
    long long basedir_sum;
    gchar supported_regex[256];
} cache_info_t;

#include "icon-module.h"

    
int mime_icon_load_theme (const gchar * theme_name);
gchar *mime_icon_get_local_xml_file (void);
gchar *mime_icon_get_global_xml_file (void);
gchar *mime_icon_get_theme_path (void);

#include "icon-module-theme.i"
#include "icon-module-mime.i"
/******************************************************************/

G_MODULE_EXPORT const gchar *
g_module_check_init (GModule * module) {
    TRACE ("ICON: g_module_check_init...\n");
    theme_name = g_strdup (theme_name);
    mime_icon_load_theme ("Rodent");
    return NULL;
}

G_MODULE_EXPORT gchar *
mime_icon_get_local_xml_file (void) {
    gchar *n,
     *mimefile;
    if(!theme_name)
        return NULL;

    n = g_build_filename (g_get_user_data_dir (), "rfm", theme_name, NULL);
    mimefile = g_strconcat (n, ".mime.xml", NULL);
    g_free (n);
    return mimefile;
}

G_MODULE_EXPORT gchar *
mime_icon_get_theme_path (void
    ) {
    gchar **p;
    gchar *theme_path = NULL;
    gchar *sdirs[] = {
        "/usr/share/icons",
        "/usr/local/share/icons",
        NULL,
        NULL
    };

    sdirs[2] = g_build_filename (PACKAGE_DATA_DIR, "icons", NULL);
    if(strcmp (sdirs[2], sdirs[0]) == 0 || strcmp (sdirs[2], sdirs[1]) == 0) {
        g_free (sdirs[2]);
        sdirs[2] = NULL;
    }

    /* literal name */
    TRACE ("ICON: mime_icon_get_theme_path...%s\n", theme_name);
    for(p = sdirs; *p; p++) {
        theme_path = g_build_filename (G_DIR_SEPARATOR_S, *p, theme_name, NULL);
        TRACE ("ICON: mime_icon_get_theme_path...%s?\n", theme_path);
        if(g_file_test (theme_path, G_FILE_TEST_IS_DIR)) {
            goto done;
        }
        g_free (theme_path);
        theme_path = NULL;
    }

    /* literal name not found: is it a translated name? */

    TRACE ("ICON: mime_icon_get_theme_path...\n");
    for(p = sdirs; *p; p++) {
        GKeyFile *key_file;
        GError *error = NULL;
        gchar *name;
        const char *file;
        GDir *gdir;

        if(!g_file_test (*p, G_FILE_TEST_IS_DIR))
            continue;
        gdir = g_dir_open (*p, 0, NULL);
        if(!gdir)
            continue;

        while((file = g_dir_read_name (gdir))) {
            gchar *themeindex = g_build_path (G_DIR_SEPARATOR_S, *p, file, "index.theme", NULL);

            key_file = g_key_file_new ();
            if(!g_key_file_load_from_file (key_file, themeindex, 0, &error)) {
                g_warning ("g_key_file_load_from_file %s: %s", themeindex, error->message);
                g_error_free (error);
                error = NULL;
                g_key_file_free (key_file);
                g_free (themeindex);
                continue;
            }
            g_free (themeindex);
            name = g_key_file_get_string (key_file, "Icon Theme", "Name", &error);
            if(error) {
                g_warning ("g_key_file_get_string: %s", error->message);
                g_key_file_free (key_file);
                g_error_free (error);
                error = NULL;
                continue;
            }
            TRACE ("ICON: name=%s\n", name);
            g_key_file_free (key_file);
            if(name && strcmp (name, theme_name) == 0) {
                /* we got a hit */
                theme_path = g_build_path (G_DIR_SEPARATOR_S, *p, file, NULL);
                if(g_file_test (theme_path, G_FILE_TEST_IS_DIR))
                    break;
                g_free (theme_path);
                theme_path = NULL;
            }
        }
        g_dir_close (gdir);
        if(theme_path)
            break;
    }
  done:
    TRACE ("ICON: mime_icon_get_global_xml_file...done\n");

    return theme_path;
}


G_MODULE_EXPORT gchar *
mime_icon_get_global_xml_file (void) {
    const gchar *themepath = NULL;
    gchar *mimefile = NULL;
    if(!theme_name) {
        g_warning ("theme_name==NULL");
        return NULL;
    }
    //TRACE("mime_icon_get_global_xml_file name...%s\n",theme_name);
    themepath = mime_icon_get_theme_path ();
    if(themepath) {
        gchar *b = g_path_get_basename (themepath);
        mimefile = g_strconcat (PACKAGE_DATA_DIR, G_DIR_SEPARATOR_S, "rfm", G_DIR_SEPARATOR_S, b, ".mime.xml", NULL);
        g_free (b);
    }

    TRACE ("ICON: mime_icon_get_global_xml_file...3:%s\n", mimefile);
    return mimefile;
}

static void
close_theme (void
    ) {
    g_free (theme_name);
    theme_name = g_strdup ("Rodent");
    // reload default theme
    char *mimefile = mime_icon_get_local_xml_file ();
    if(!mimefile || !g_file_test (mimefile, G_FILE_TEST_EXISTS)) {
        mimefile = mime_icon_get_global_xml_file ();
    }
    if(mimefile) {
        g_warning ("using %s as mimefile", mimefile);
    }
}

void
g_module_unload (GModule * module) {
    close_theme ();
    g_free (theme_name);
    theme_name = NULL;
}

G_MODULE_EXPORT int
mime_icon_load_theme (const gchar * input_theme_name) {
    static gboolean theme_open=FALSE;
    if (theme_open) return 1;
    // this is to create a cache for full paths for icons
    if(open_theme (input_theme_name) < 0) {
        g_warning ("mime_icon_load_theme(): cannot load theme %s", input_theme_name);
        if(open_theme ("Rodent") < 0) {
            g_warning ("mime_icon_load_theme(): cannot load theme Rodent");
            return -1;
        }
    }
    theme_open=TRUE;
    // this is to create a g_hash which will associate mimetype with an icon
    // the mimetype/icon association hash is created by parsing the xml file.
    gchar *mimefile = mime_icon_get_local_xml_file ();
    TRACE ("ICONk: looking for: %s\n", (mimefile ? mimefile : "null"));

    /* test for default mimefile  */
    if(!mimefile || !g_file_test (mimefile, G_FILE_TEST_EXISTS)) {
        g_free (mimefile);
        mimefile = mime_icon_get_global_xml_file ();
        TRACE ("ICONk: not found. Now looking for: %s\n", (mimefile ? mimefile : "null"));
    }

    if(!mimefile || !g_file_test (mimefile, G_FILE_TEST_EXISTS)) {
        g_warning ("No mime theme found for %s", (theme_name ? theme_name : "null"));
        g_free (mimefile);
        mimefile = NULL;
    }

    TRACE ("ICONk: icon-module: icon_theme_name=%s\n", (theme_name ? theme_name : "null"));

    int r = create_icon_hash (mimefile);
    g_free (mimefile);
    TRACE ("ICONk: mime_icon_load_theme done\n");
    if(r)
        return r;
    g_warning ("cannot create icon_hash from mimefile:%s!", (mimefile ? mimefile : "null"));
    return 0;
}

G_MODULE_EXPORT void *
module_init (void) {
    return NULL;
}

G_MODULE_EXPORT gchar *
mime_icon_get_filename_from_id (const gchar * id) {
    gchar *basename;
    TRACE ("ICON: mime_icon_get_filename_from_id\n");
    // look for basename path in hashtable...
    basename = ((gchar *) g_hash_table_lookup (icon_hash, id));
    
    if(!basename) {
        if(!strchr (id, '/'))
            return NULL;
        gchar *g = g_strdup (id);
        *strchr (g, '/') = 0;
        gchar *gg = g_strconcat (g, "/", "default", NULL);
        basename = (gchar *)g_hash_table_lookup (icon_hash, gg);
        g_free (g);
        g_free (gg);
    }
    if(!basename) {
        DBG ("cannot get icon filename for id=%s\n", id);
        return NULL;
    }
    // look for full path in dbh cache...
    
    gchar *file = mime_icon_find_pixmap_file (basename);
    if(!file || !g_file_test (file, G_FILE_TEST_EXISTS)) {
        DBG ("mime_icon_get_filename_from_id(): %s->%s does not exist\n", id, file);
    }

    return (file);
}


